# %CopyrightBegin%
#
# SPDX-License-Identifier: Apache-2.0
# 
# Copyright Ericsson AB 2000-2025. All Rights Reserved.
# 
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# 
# %CopyrightEnd%

s/Integer32/INTEGER/g
s/Counter32/Counter/g
s/Gauge32/Gauge/g
s/MAX-ACCESS/ACCESS    /g
s/OBJECTS/VARIABLES/g
s/current[^a-zA-Z0-9]/mandatory/g
s/current$/mandatory/g
s/accessible-for-notify/not-accessible/g
s/read-create/read-write/g
