%%
%% %CopyrightBegin%
%%
%% SPDX-License-Identifier: Apache-2.0
%%
%% Copyright Ericsson AB 2026. All Rights Reserved.
%%
%% Licensed under the Apache License, Version 2.0 (the "License");
%% you may not use this file except in compliance with the License.
%% You may obtain a copy of the License at
%%
%%     http://www.apache.org/licenses/LICENSE-2.0
%%
%% Unless required by applicable law or agreed to in writing, software
%% distributed under the License is distributed on an "AS IS" BASIS,
%% WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
%% See the License for the specific language governing permissions and
%% limitations under the License.
%%
%% %CopyrightEnd%
%%

-module(ct_doctest_prompt_parser_mod).
-moduledoc """
```
%% Comment before first prompt
1> A = 1,
  A + 2.
3
2> [1, 2].
[
 1
 ,
 2
 ]
3> [1,
  %% Comment between prompts
  2].
[1,
 %% Comment in match
 2]
4> [1,
  2].
[1,
 2]
5> 1 + 2. %% Comment after prompt with .
3 %% Comment after match
%% Comment after last prompt
```
""".

-export([f/0, g/0, h/0]).

-doc """
Incorrect first prompt number

```
2> 1+2.
```
""".
f() ->
    ok.

-doc """
Incorrect first prompt number with comment

```
%% Comment before prompt
2> 1+2.
```
""".
g() ->
    ok.

-doc """
Incorrect second prompt number

```
1> 1+2.
3
3> 1+2.
```

""".
h() ->
    ok.